// eslint-disable-next-line jest/no-commented-out-tests
const chai = require('chai');
const sinon = require('sinon');
const chaiAsPromised = require('chai-as-promised');
const UID = require('./../../../../routes/arrayfunctions').getUUID;

const { ZoneFactory } = require('./utils');
const AddonZones = require('../index');
const { ZoneRepo } = require('../../repository/zones');

chai.use(chaiAsPromised);
const { expect } = chai;

const invalidArgumentsList = [undefined, null, false, {}, NaN, []];
const zoneFactory = new ZoneFactory();
let ZoneRepo_getZoneBySequenceAndProject;
let ZoneRepo_updateCoupledZone;
let ZoneRepo_getZoneByUid;


describe('Boundary DB methods functions', () => {
    beforeEach(() => {
        ZoneRepo_getZoneBySequenceAndProject = sinon.stub(ZoneRepo, 'getZoneBySequenceAndProject');
        ZoneRepo_updateCoupledZone = sinon.stub(ZoneRepo, 'updateCoupledZone');
        ZoneRepo_getZoneByUid = sinon.stub(ZoneRepo, 'getZoneByUid');
    });
    
    afterEach(() => {
        ZoneRepo_getZoneBySequenceAndProject.restore();
        ZoneRepo_updateCoupledZone.restore();
        ZoneRepo_getZoneByUid.restore();
    });
    
    
    describe('getZoneById', () => {
        it('it should return defined zone by id', async () => {
            const zone = zoneFactory.createKeepInZone();
            const zone_id = 2;
            const project_uid = UID();
            zone.zone_id = zone_id;
            ZoneRepo_getZoneBySequenceAndProject.withArgs(zone_id, project_uid).returns(zone);
            
            const dbZone = await AddonZones.getZoneById(zone_id, project_uid);

            expect(ZoneRepo_getZoneBySequenceAndProject.callCount).to.equal(1);
            expect(ZoneRepo_updateCoupledZone.callCount).to.equal(0);
            expect(ZoneRepo_getZoneByUid.callCount).to.equal(0);
            expect(dbZone).to.be.a('object');
            expect(dbZone).to.deep.equal(dbZone);
        });
    
        it('it should return null if zone_id or project_uid is invalid and not execute db methods', async () => {
            const zone_id = 2;
            const project_uid = UID();

            for (const invalidArgument of invalidArgumentsList) {
                const dbZoneOne = await AddonZones.getZoneById(invalidArgument, project_uid);
                const dbZoneTwo = await AddonZones.getZoneById(zone_id, invalidArgument);
                expect(dbZoneOne).to.equal(null);
                expect(dbZoneTwo).to.equal(null);
            }

            expect(ZoneRepo_getZoneBySequenceAndProject.callCount).to.equal(0);
            expect(ZoneRepo_updateCoupledZone.callCount).to.equal(0);
            expect(ZoneRepo_getZoneByUid.callCount).to.equal(0);
        });
    });
    
    describe('updateCoupledZone', () => {
        it('it should call db update method', async () => {
            const coupled = 1;
            const zoneUid = UID();

            await AddonZones.updateCoupledZone(zoneUid, coupled);

            expect(ZoneRepo_getZoneBySequenceAndProject.callCount).to.equal(0);
            expect(ZoneRepo_updateCoupledZone.withArgs(coupled, zoneUid).callCount).to.equal(1);
            expect(ZoneRepo_getZoneByUid.callCount).to.equal(0);
        });
    
        it('it should not call db update method if one of arguments is invalid', async () => {
            const coupled = 1;
            const zoneUid = UID();
        
            for (const invalidArgument of invalidArgumentsList) {
                await AddonZones.updateCoupledZone(invalidArgument, coupled);
                await AddonZones.updateCoupledZone(zoneUid, invalidArgument);
                await AddonZones.updateCoupledZone(invalidArgument, invalidArgument);
            }

            expect(ZoneRepo_getZoneBySequenceAndProject.callCount).to.equal(0);
            expect(ZoneRepo_updateCoupledZone.withArgs(coupled, zoneUid).callCount).to.equal(0);
            expect(ZoneRepo_getZoneByUid.callCount).to.equal(0);
        });
    });
    
    describe('getZoneByUid', () => {
        it('should return zone by uid', async () => {
            const zoneUid = UID();

            await AddonZones.getZoneByUid(zoneUid);

            expect(ZoneRepo_getZoneBySequenceAndProject.callCount).to.equal(0);
            expect(ZoneRepo_updateCoupledZone.callCount).to.equal(0);
            expect(ZoneRepo_getZoneByUid.withArgs(zoneUid).callCount).to.equal(1);
        });
    
        it('should return null when invalid arguments', async () => {
            for (const invalidArgument of invalidArgumentsList) {
                const zone = await AddonZones.getZoneByUid(invalidArgument);
                expect(zone).to.equal(null);
            }

            expect(ZoneRepo_getZoneBySequenceAndProject.callCount).to.equal(0);
            expect(ZoneRepo_updateCoupledZone.callCount).to.equal(0);
            expect(ZoneRepo_getZoneByUid.callCount).to.equal(0);
        });
    });
});
