const chai = require('chai');
const chaiAsPromised = require('chai-as-promised');
const { USER_ROLES_VALUES } = require('enums').COMMON;
const { ZoneFactory, ZoneListFactory, UserFactory } = require('./utils');
const { BOUNDARY_ZONES } = require('../../constants');

const {
    getZonePrototypesBlackList, filterZonesByUserRole, isBoundaryZone, isCoupledZone,
    isCoupledBoundaryZone, isNeedReCouple,
} = require('../../utils');

chai.use(chaiAsPromised);
const { expect } = chai;

const default_black_list = [BOUNDARY_ZONES.BOUNDARY_OUT, BOUNDARY_ZONES.BOUNDARY_IN];
const invalidArgumentsList = [undefined, null, false, {}, NaN, [], 0];

const userFactory = new UserFactory();
const zoneFactory = new ZoneFactory();
const zoneListFactory = new ZoneListFactory();


describe('Boundary Zones Utils Functions', () => {
    describe('getZonePrototypesBlackList', () => {
        it('it should return empty list for non defined user_role', () => {
            const black_list = getZonePrototypesBlackList();
            expect(black_list).to.be.a('array');
            expect(black_list.length).to.equal(0);
        });
    
        it('it should return empty list for not existed user_role', () => {
            const black_list = getZonePrototypesBlackList('non-existed-role');
            expect(black_list).to.be.a('array');
            expect(black_list.length).to.equal(0);
        });
    
        it('it should return boundary (in/out) zones list for project users', () => {
            const project_user_black_list = getZonePrototypesBlackList(USER_ROLES_VALUES.PROJECT_USER);
            const project_system_admin_black_list = getZonePrototypesBlackList(USER_ROLES_VALUES.PROJECT_SYSTEM_ADMIN);
            const project_operations_admin_black_list = getZonePrototypesBlackList(USER_ROLES_VALUES.PROJECT_OPERATIONS_ADMIN);
            
            expect(project_user_black_list).to.be.a('array');
            expect(project_user_black_list.length).to.equal(2);
            expect(project_user_black_list).to.deep.equal(default_black_list);
    
            expect(project_system_admin_black_list).to.be.a('array');
            expect(project_system_admin_black_list.length).to.equal(2);
            expect(project_system_admin_black_list).to.deep.equal(default_black_list);
    
            expect(project_operations_admin_black_list).to.be.a('array');
            expect(project_operations_admin_black_list.length).to.equal(2);
            expect(project_operations_admin_black_list).to.deep.equal(default_black_list);
        });
    
        it('it should return empty list for company admin', () => {
            const company_admin_black_list = getZonePrototypesBlackList(USER_ROLES_VALUES.COMPANY_ADMIN);
            expect(company_admin_black_list).to.be.a('array');
            expect(company_admin_black_list.length).to.equal(0);
        });
    });
    
    describe('filterZonesByUserRole', () => {
        it('should not filter the list for undefined user', () => {
            const zones = zoneListFactory.createZonesListByLength(2);
            const filtered_list = filterZonesByUserRole(undefined, zones);
            
            expect(filtered_list).to.be.a('array');
            expect(filtered_list.length).to.equal(zones.length);
            expect(filtered_list).to.deep.equal(zones);
        });
    
        it('should not filter the list for unknown user role', () => {
            const user = userFactory.createUnknownRoleUser();
            const zones = zoneListFactory.createZonesListByLength();
            const filtered_list = filterZonesByUserRole(user, zones);
        
            expect(filtered_list).to.be.a('array');
            expect(filtered_list.length).to.equal(zones.length);
            expect(filtered_list).to.deep.equal(zones);
        });
    
        it('should not filter the list for company admin user', () => {
            const user = userFactory.createCompanyAdmin();
            const zones = zoneListFactory.createZonesListByLength();
            const filtered_list = filterZonesByUserRole(user, zones);
    
            expect(user.role).to.equal(USER_ROLES_VALUES.COMPANY_ADMIN);
            expect(filtered_list).to.be.a('array');
            expect(filtered_list.length).to.equal(zones.length);
            expect(filtered_list).to.deep.equal(zones);
        });
    
        it('should not filter boundary zones for the project user', () => {
            const user = userFactory.createProjectUser();
            const zones = zoneListFactory.createZonesListByLength();
            const noBoundaryZones = zoneListFactory.createNoBoundaryZonesList();
            const filtered_list = filterZonesByUserRole(user, zones);
    
            expect(user.role).to.equal(USER_ROLES_VALUES.PROJECT_USER);
            expect(filtered_list).to.be.a('array');
            expect(filtered_list.length).to.equal(noBoundaryZones.length);
            const boundaryZone = filtered_list.find(zone => zone.zone_type_name === BOUNDARY_ZONES.BOUNDARY_OUT || zone.zone_type_name === BOUNDARY_ZONES.BOUNDARY_IN);
            expect(boundaryZone).to.be.undefined;
        });
    
        it('should not filter boundary zones for the project operations admin user', () => {
            const user = userFactory.createProjectOperationsAdmin();
            const zones = zoneListFactory.createZonesListByLength();
            const noBoundaryZones = zoneListFactory.createNoBoundaryZonesList();
            const filtered_list = filterZonesByUserRole(user, zones);
    
            expect(user.role).to.equal(USER_ROLES_VALUES.PROJECT_OPERATIONS_ADMIN);
            expect(filtered_list).to.be.a('array');
            expect(filtered_list.length).to.equal(noBoundaryZones.length);
            const boundaryZone = filtered_list.find(zone => zone.zone_type_name === BOUNDARY_ZONES.BOUNDARY_OUT || zone.zone_type_name === BOUNDARY_ZONES.BOUNDARY_IN);
            expect(boundaryZone).to.be.undefined;
        });
    
        it('should not filter boundary zones for the project system admin user', () => {
            const user = userFactory.createProjectSystemAdmin();
            const zones = zoneListFactory.createZonesListByLength();
            const noBoundaryZones = zoneListFactory.createNoBoundaryZonesList();
            const filtered_list = filterZonesByUserRole(user, zones);
    
            expect(user.role).to.equal(USER_ROLES_VALUES.PROJECT_SYSTEM_ADMIN);
            expect(filtered_list).to.be.a('array');
            expect(filtered_list.length).to.equal(noBoundaryZones.length);
            const boundaryZone = filtered_list.find(zone => zone.zone_type_name === BOUNDARY_ZONES.BOUNDARY_OUT || zone.zone_type_name === BOUNDARY_ZONES.BOUNDARY_IN);
            expect(boundaryZone).to.be.undefined;
        });
    
        it('should handle not valid zones argument', () => {
            const user = userFactory.createProjectSystemAdmin();
            const undefinedValue = undefined;
            const nullValue = null;
            const boolValue = true;
            const stringValue = 'some string';
            const objectValue = { };
            
            const filtered_undef = filterZonesByUserRole(user, undefinedValue);
            const filtered_null = filterZonesByUserRole(user, nullValue);
            const filtered_bool = filterZonesByUserRole(user, boolValue);
            const filtered_string = filterZonesByUserRole(user, stringValue);
            const filtered_object = filterZonesByUserRole(user, objectValue);
        
            expect(filtered_undef).to.equal(undefinedValue);
            expect(filtered_null).to.equal(nullValue);
            expect(filtered_bool).to.equal(boolValue);
            expect(filtered_string).to.equal(stringValue);
            expect(filtered_object).to.equal(objectValue);
        });
    });
    
    describe('isBoundaryZone', () => {
        it('should handle any type of arguments', () => {
            for (const invalidArgument of invalidArgumentsList) {
                expect(isBoundaryZone(invalidArgument)).to.equal(false);
            }
            expect(isBoundaryZone(BOUNDARY_ZONES.BOUNDARY_IN)).to.equal(true);
            expect(isBoundaryZone(BOUNDARY_ZONES.BOUNDARY_OUT)).to.equal(true);
        });
    });
    
    describe('isCoupledZone', () => {
        it('should handle any type of arguments', () => {
            const coupleId = 2;
            const keepInZone = zoneFactory.createKeepInZone();
            const keepInZoneCoupled = zoneFactory.createKeepInZone(coupleId);
            const boundaryOutZoneNoCoupled = zoneFactory.createBoundaryOutZone();
            const boundaryInZoneNoCoupled = zoneFactory.createBoundaryInZone();
            const boundaryOutZoneCoupled = zoneFactory.createBoundaryOutZone(coupleId);
            const boundaryInZoneCoupled = zoneFactory.createBoundaryInZone(coupleId);
    
            for (const invalidArgument of invalidArgumentsList) {
                expect(isCoupledZone(invalidArgument)).to.equal(false);
            }
            expect(isCoupledZone(keepInZone)).to.equal(false);
            expect(isCoupledZone(boundaryInZoneNoCoupled)).to.equal(false);
            expect(isCoupledZone(boundaryOutZoneNoCoupled)).to.equal(false);
    
            expect(isCoupledZone(keepInZoneCoupled)).to.equal(true);
            expect(isCoupledZone(boundaryInZoneCoupled)).to.equal(true);
            expect(isCoupledZone(boundaryOutZoneCoupled)).to.equal(true);
        });
    });
    
    describe('isCoupledBoundaryZone', () => {
        it('should handle any type of arguments', () => {
            const coupleId = 2;
            const keepInZone = zoneFactory.createKeepInZone();
            const keepInZoneCoupled = zoneFactory.createKeepInZone(coupleId);
            const boundaryOutZoneNoCoupled = zoneFactory.createBoundaryOutZone();
            const boundaryInZoneNoCoupled = zoneFactory.createBoundaryInZone();
            const boundaryOutZoneCoupled = zoneFactory.createBoundaryOutZone(coupleId);
            const boundaryInZoneCoupled = zoneFactory.createBoundaryInZone(coupleId);
    
            for (const invalidArgument of invalidArgumentsList) {
                expect(isCoupledBoundaryZone(invalidArgument)).to.equal(false);
            }
            expect(isCoupledBoundaryZone(keepInZone)).to.equal(false);
            expect(isCoupledBoundaryZone(boundaryInZoneNoCoupled)).to.equal(false);
            expect(isCoupledBoundaryZone(boundaryOutZoneNoCoupled)).to.equal(false);
            expect(isCoupledBoundaryZone(keepInZoneCoupled)).to.equal(false);
            
            expect(isCoupledBoundaryZone(boundaryInZoneCoupled)).to.equal(true);
            expect(isCoupledBoundaryZone(boundaryOutZoneCoupled)).to.equal(true);
        });
    });
    
    describe('isNeedReCouple', () => {
        it('should return false if zone not change couple id', () => {
            const coupleId = 1;
            const boundaryOutZoneCoupled = zoneFactory.createBoundaryOutZone(coupleId);
            const boundaryInZoneCoupled = zoneFactory.createBoundaryInZone(coupleId);
            
            expect(isNeedReCouple(boundaryOutZoneCoupled, boundaryOutZoneCoupled)).to.equal(false);
            expect(isNeedReCouple(boundaryInZoneCoupled, boundaryInZoneCoupled)).to.equal(false);
        });
    
        it('should return false if zones in arguments has different uids', () => {
            const coupleIdOne = 1;
            const coupleIdTwo = 2;
            const boundaryOutZoneCoupledOne = zoneFactory.createBoundaryOutZone(coupleIdOne);
            const boundaryOutZoneCoupledTwo = zoneFactory.createBoundaryOutZone(coupleIdTwo);
            const boundaryInZoneCoupledOne = zoneFactory.createBoundaryInZone(coupleIdOne);
            const boundaryInZoneCoupledTwo = zoneFactory.createBoundaryInZone(coupleIdTwo);
        
            expect(isNeedReCouple(boundaryOutZoneCoupledOne, boundaryOutZoneCoupledTwo)).to.equal(false);
            expect(isNeedReCouple(boundaryInZoneCoupledOne, boundaryInZoneCoupledTwo)).to.equal(false);
        });
    
        it('should return true if zone change couple id', () => {
            const coupleIdOne = 1;
            const coupleIdTwo = 2;
            const boundaryOutZoneCoupled = zoneFactory.createBoundaryOutZone(coupleIdOne);
            const boundaryOutZoneCoupledOne = boundaryOutZoneCoupled.getProperties();
            boundaryOutZoneCoupled.setCoupleId(coupleIdTwo);
            const boundaryOutZoneCoupledTwo = boundaryOutZoneCoupled.getProperties();
            
            const boundaryInZoneCoupled = zoneFactory.createBoundaryInZone(coupleIdOne);
            const boundaryInZoneCoupledOne = boundaryInZoneCoupled.getProperties();
            boundaryInZoneCoupled.setCoupleId(coupleIdTwo);
            const boundaryInZoneCoupledTwo = boundaryInZoneCoupled.getProperties();
            
            expect(isNeedReCouple(boundaryOutZoneCoupledOne, boundaryOutZoneCoupledTwo)).to.equal(true);
            expect(isNeedReCouple(boundaryInZoneCoupledOne, boundaryInZoneCoupledTwo)).to.equal(true);
        });
    
        it('should return false if it is not boundary zone', () => {
            const coupleIdOne = 1;
            const coupleIdTwo = 2;
            const keepInZoneCoupled = zoneFactory.createKeepInZone(coupleIdOne);
            const keepInZoneCoupledOne = keepInZoneCoupled.getProperties();
            keepInZoneCoupled.setCoupleId(coupleIdTwo);
            const keepInZoneCoupledTwo = keepInZoneCoupled.getProperties();
        
            expect(isNeedReCouple(keepInZoneCoupledOne, keepInZoneCoupledTwo)).to.equal(false);
        });
    
        it('should handle invalid arguments and return false', () => {
            const coupleId = 1;
            const boundaryOutZoneCoupled = zoneFactory.createBoundaryOutZone(coupleId);

            for (const invalidArgument of invalidArgumentsList) {
                expect(isNeedReCouple(boundaryOutZoneCoupled, invalidArgument)).to.equal(false);
                expect(isNeedReCouple(invalidArgument, boundaryOutZoneCoupled)).to.equal(false);
                expect(isNeedReCouple(invalidArgument, invalidArgument)).to.equal(false);
            }
        });
    });
});
