const error = require('http-errors');
const logger = require('../../../utils/logger/index');
const CoreZonesService = require('../../../routes/zones/service');
const CoreZones = require('../../../routes/zones/index');
const { formatZone } = require('../../../routes/zones/utils');
const ZMQUtils = require('./../../../routes/utils_zmq');
const AddonZones = require('./index');
const { isCoupledBoundaryZone, isBoundaryZone, isNeedReCouple, filterZonesByUserRole } = require('../utils');
const { USER_ROLES_VALUES } = require('enums').COMMON;


async function notifyUpdatedZone(req, zone_uid) {
    const updatedZone = formatZone(await CoreZones.getZones(req.projectTag, zone_uid.uid).get(0));
    ZMQUtils.notify({
        req,
        object: ZMQUtils.OBJECT_TYPE.ZMQ_OBJ.ZONE,
        action: ZMQUtils.OBJECT_TYPE.TYPE_ACTION.UPDATED,
        plist: updatedZone,
    });
}

async function coupleZone(req, zone_id, couple_id) {
    const zone = await AddonZones.getZoneById(zone_id, req.projectTag.uid);
    if (!zone) {
        return;
    }
    await AddonZones.coupleZone(zone.uid, couple_id);
    await notifyUpdatedZone(req, zone);
}

async function decoupleZone(req, zone_id) {
    const zone = await AddonZones.getZoneById(zone_id, req.projectTag.uid);
    if (!zone) {
        return;
    }
    await AddonZones.decoupleZone(zone.uid);
    await notifyUpdatedZone(req, zone);
}

async function handleCoupledZone(req, zone, zoneBeforeUpdate) {
    const needCoupleRelatedZone = isCoupledBoundaryZone(zone) && !isCoupledBoundaryZone(zoneBeforeUpdate);
    const needDecoupleRelatedZone = !isCoupledBoundaryZone(zone) && isCoupledBoundaryZone(zoneBeforeUpdate);
    // If updated zone has coupling with another zone it is necessary decouple prev coupled zone and couple new one
    const needRecoupleZones = isNeedReCouple(zone, zoneBeforeUpdate);
    
    if (needCoupleRelatedZone) {
        const zone_id = zone.user_params.coupled;
        const couple_id = zone.zone_id;
        
        await coupleZone(req, zone_id, couple_id);
    } else if (needDecoupleRelatedZone) {
        const zone_id = zoneBeforeUpdate.user_params.coupled;
        
        await decoupleZone(req, zone_id);
    } else if (needRecoupleZones) {
        const zone_id = zone.user_params.coupled;
        const couple_id = zone.zone_id;
        const decouple_id = zoneBeforeUpdate.user_params.coupled;
        
        await coupleZone(req, zone_id, couple_id);
        await decoupleZone(req, decouple_id);
    }
}

async function getZones(req) {
    const zones = await CoreZonesService.getZones(req);
    return filterZonesByUserRole(req.user, zones);
}

function checkUserRole(req, next, zone) {
    let zone_type_name = req.body && (req.body.zone || req.body).zone_type_name;
    if (zone) {
        zone_type_name = zone.zone_type_name;
    }
    if (req.user.role !== USER_ROLES_VALUES.COMPANY_ADMIN
        && isBoundaryZone(zone_type_name)) {
        next(error(403, 'not admin'));
    }
}

async function createZone(req, next) {
    try {
        checkUserRole(req, next);
        const createdZone = await CoreZonesService.createZone(req, next);
        await handleCoupledZone(req, createdZone);
        
        return createdZone;
    } catch (e) {
        logger.error('Boundary Zone Addon [createZone]', e);
        next(e);
    }
}

async function updateZone(req, next) {
    try {
        const zoneBeforeUpdate = await AddonZones.getZoneByUid(req.params.uid);
        checkUserRole(req, next, zoneBeforeUpdate);
        const updatedZone = await CoreZonesService.updateZone(req, next);
        await handleCoupledZone(req, updatedZone, zoneBeforeUpdate);
        
        return updatedZone;
    } catch (e) {
        logger.error('Boundary Zone Addon [updateZone]', e);
        next(e);
    }
}

async function deleteZone(req, next) {
    try {
        const removedZone = await AddonZones.getZoneByUid(req.params.uid);
        checkUserRole(req, next, removedZone);
        await CoreZonesService.deleteZone(req);

        if (removedZone && removedZone.user_params && removedZone.user_params.coupled) {
            const zone_to_decouple = removedZone.user_params.coupled;
            if (zone_to_decouple) {
                await decoupleZone(req, zone_to_decouple);
            }
        }
    } catch (e) {
        logger.error('Boundary Zone Addon [deleteZone]', e);
        next(e);
    }
}


module.exports = {
    deleteZone,
    createZone,
    updateZone,
    getZones,
};
